
//ResizeControl.cpp

#include "stdafx.h"
#include "TestDlg.h"
#include "ResizeControl.h"

#define UWM_CONTROLRESIZE_MSG _T("UWM_CONTROLRESIZE_MSG-{D55C7085_FC4C_138A_BD72_01D0293651C0}")

//Resize Message
const UINT IResizeControl::UWM_CONTROLRESIZE = ::RegisterWindowMessage(UWM_CONTROLRESIZE_MSG);

//Mouse Cursors
HCURSOR IResizeControl::sm_hWE = ::AfxGetApp()->LoadStandardCursor(IDC_SIZEWE);
HCURSOR IResizeControl::sm_hNS = ::AfxGetApp()->LoadStandardCursor(IDC_SIZENS);
HCURSOR IResizeControl::sm_hNWSE = ::AfxGetApp()->LoadStandardCursor(IDC_SIZENWSE);
HCURSOR IResizeControl::sm_hNESW = ::AfxGetApp()->LoadStandardCursor(IDC_SIZENESW);
HCURSOR IResizeControl::sm_hDEF = ::AfxGetApp()->LoadStandardCursor(IDC_ARROW);

//CONSTRUCTOR
IResizeControl::IResizeControl(bool bN, bool bW, bool bS, bool bE, int iMinWidth,
	int iMaxWidth, int iMinHeight, int iMaxHeight, bool bNotify) : m_bN(bN), m_bW(bW), m_bS(bS), m_bE(bE),
	m_iMinWidth(iMinWidth), m_iMaxWidth(iMaxWidth), m_iMinHeight(iMinHeight),
	m_iMaxHeight(iMaxHeight), m_iPosition(POSDEF), m_bTrack(false), m_bNotify(bNotify)
{
	//Check the limits
	ASSERT((iMinWidth>0)&&(iMaxWidth>0)&&(iMinHeight>0)&&(iMaxHeight>0)&&
		(iMinWidth<iMaxWidth)&&(iMinHeight<iMaxHeight));
}

//DESTRUCTOR
IResizeControl::~IResizeControl()
{
}

int IResizeControl::FindPosition(POINT const& rPt, CRect const& roRect)
{
	int iX = GetSystemMetrics(SM_CXSIZEFRAME);
	int iY = GetSystemMetrics(SM_CYSIZEFRAME);
	int iPosition = POSDEF;
	if(abs(rPt.x-roRect.left)<iX)
	{
		//Left Side
		if(abs(rPt.y-roRect.top)<iY)
		{
			//Left Top
			if(true==m_bN && true==m_bW)
				iPosition = POSNW;
		}
		else if(abs(rPt.y-roRect.bottom)<iY)
		{
			//Left Bottom
			if(true==m_bS && true==m_bW)
				iPosition = POSSW;
		}
		else
		{
			if(true==m_bW)
				iPosition = POSW;
		}
	}
	else if(abs(rPt.x-roRect.right)<iX)
	{
		//Right Side
		if(abs(rPt.y-roRect.top)<iY)
		{
			//Right Top
			if(true==m_bN && true==m_bE)
				iPosition = POSNE;
		}
		else if(abs(rPt.y-roRect.bottom)<iY)
		{
			//Right Bottom
			if(true==m_bS && true==m_bE)
				iPosition = POSSE;
		}
		else
		{
			if(true==m_bE)
				iPosition = POSE;
		}
	}
	else if(abs(rPt.y-roRect.top)<iY)
	{
		if(true==m_bN)
			iPosition = POSN;
	}
	else if(abs(rPt.y-roRect.bottom)<iY)
	{
		if(true==m_bS)
			iPosition = POSS;
	}
	return iPosition;
}

//Determine the new Dimensions
void IResizeControl::NewDimensions(POINT const& rPt, CRect const& roRect, int& riLeft,
	int& riTop, int& riWidth, int& riHeight, bool& rbResize)
{
	rbResize = false;
	switch(m_iPosition)
	{
		case POSN:
			//Check new height
			riHeight = roRect.bottom-rPt.y;
			if((riHeight>=m_iMinHeight) && (riHeight<=m_iMaxHeight))
			{
				rbResize = true;
				riLeft = roRect.left;
				riTop = rPt.y;
				riWidth = roRect.Width();
			}
			break;

		case POSNW:
		{
			bool bResizeH=true, bResizeV=true;
			//Check new width and height
			riWidth = roRect.right-rPt.x;
			if((riWidth<m_iMinWidth) || (riWidth>m_iMaxWidth))
			{
				bResizeH = false;
				riWidth = roRect.Width();
			}
			else
				rbResize = true;
			riHeight = roRect.bottom-rPt.y;
			if((riHeight<m_iMinHeight) || (riHeight>m_iMaxHeight))
			{
				bResizeV = false;
				riHeight = roRect.Height();
			}
			else
				rbResize = true;
			if(true == rbResize)
			{
				if(true == bResizeH)
					riLeft = rPt.x;
				else
					riLeft = roRect.left;
				if(true == bResizeV)
					riTop = rPt.y;
				else
					riTop = roRect.top;
			}
			break;
		}

		case POSW:
			//Check new width
			riWidth = roRect.right-rPt.x;
			if((riWidth>=m_iMinWidth) && (riWidth<=m_iMaxWidth))
			{
				rbResize = true;
				riLeft = rPt.x;
				riTop = roRect.top;
				riHeight = roRect.Height();
			}
			break;
			
		case POSSW:
		{
			bool bResizeH=true;
			//Check new width and height
			riWidth = roRect.right-rPt.x;
			if((riWidth<m_iMinWidth) || (riWidth>m_iMaxWidth))
			{
				bResizeH=false;
				riWidth = roRect.Width();
			}
			else
				rbResize = true;
			riHeight = rPt.y-roRect.top;
			if((riHeight<m_iMinHeight) || (riHeight>m_iMaxHeight))
				riHeight = roRect.Height();
			else
				rbResize = true;
			if(true == rbResize)
			{
				if(true==bResizeH)
					riLeft = rPt.x;
				else
					riLeft = roRect.left;
				riTop = roRect.top;
			}
			break;
		}
		
		case POSS:
			//Check new height
			riHeight = rPt.y-roRect.top;
			if((riHeight>=m_iMinHeight) && (riHeight<=m_iMaxHeight))
			{
				rbResize = true;
				riLeft = roRect.left;
				riTop = roRect.top;
				riWidth = roRect.Width();
			}
			break;
		
		case POSSE:
			//Check new width and height
			riWidth = rPt.x-roRect.left;
			if((riWidth<m_iMinWidth) || (riWidth>m_iMaxWidth))
				riWidth = roRect.Width();
			else
				rbResize = true;
			riHeight = rPt.y-roRect.top;
			if((riHeight<m_iMinHeight) || (riHeight>m_iMaxHeight))
				riHeight = roRect.Height();
			else
				rbResize = true;
			if(true == rbResize)
			{
				riLeft = roRect.left;
				riTop = roRect.top;
			}
			break;
		
		case POSE:
			//Check new width
			riWidth = rPt.x-roRect.left;
			if((riWidth>=m_iMinWidth) && (riWidth<=m_iMaxWidth))
			{
				rbResize = true;
				riLeft = roRect.left;
				riTop = roRect.top;
				riHeight = roRect.Height();
			}
			break;
		
		case POSNE:
		{
			bool bResizeV=true;
			//Check new width and height
			riWidth = rPt.x-roRect.left;
			if((riWidth<m_iMinWidth) || (riWidth>m_iMaxWidth))
				riWidth = roRect.Width();
			else
				rbResize = true;
			riHeight = roRect.bottom-rPt.y;
			if((riHeight<m_iMinHeight) || (riHeight>m_iMaxHeight))
			{
				bResizeV=false;
				riHeight = roRect.Height();
			}
			else
				rbResize = true;
			if(true == rbResize)
			{
				riLeft = roRect.left;
				if(true == bResizeV)
					riTop = rPt.y;
				else
					riTop = roRect.top;
			}
			break;
		}
	}
}

